



pip check
IF %ERRORLEVEL% NEQ 0 exit /B 1
cd tests && npm install
IF %ERRORLEVEL% NEQ 0 exit /B 1
cd .. && pytest -vv --asyncio-mode=auto --ignore tests/test_ypy_yjs.py
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
